@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2021 Derick Rethans                               |
+   | Copyright (c) 2002-2023 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -67,10 +67,12 @@ typedef struct xdebug_var_name {
 
 #define XDEBUG_MAX_FUNCTION_LEN 1024
 
-#define XDEBUG_TRACE_OPTION_APPEND         1
-#define XDEBUG_TRACE_OPTION_COMPUTERIZED   2
-#define XDEBUG_TRACE_OPTION_HTML           4
-#define XDEBUG_TRACE_OPTION_NAKED_FILENAME 8
+#define XDEBUG_TRACE_OPTION_APPEND          0x01
+#define XDEBUG_TRACE_OPTION_COMPUTERIZED    0x02
+#define XDEBUG_TRACE_OPTION_HTML            0x04
+#define XDEBUG_TRACE_OPTION_NAKED_FILENAME  0x08
+#define XDEBUG_TRACE_OPTION_FLAMEGRAPH_COST 0x10
+#define XDEBUG_TRACE_OPTION_FLAMEGRAPH_MEM  0x20
 
 #define XDEBUG_CC_OPTION_UNUSED          1
 #define XDEBUG_CC_OPTION_DEAD_CODE       2
@@ -93,38 +95,11 @@ typedef struct xdebug_var_name {
 #define XDEBUG_CMDLOOP_NONBAIL                       0
 #define XDEBUG_CMDLOOP_BAIL                          1
 
-#define XDEBUG_ERROR_OK                              0
-#define XDEBUG_ERROR_PARSE                           1
-#define XDEBUG_ERROR_DUP_ARG                         2
-#define XDEBUG_ERROR_INVALID_ARGS                    3
-#define XDEBUG_ERROR_UNIMPLEMENTED                   4
-#define XDEBUG_ERROR_COMMAND_UNAVAILABLE             5
-
-#define XDEBUG_ERROR_CANT_OPEN_FILE                100
-#define XDEBUG_ERROR_STREAM_REDIRECT_FAILED        101 /* unused */
-
-#define XDEBUG_ERROR_BREAKPOINT_NOT_SET            200
-#define XDEBUG_ERROR_BREAKPOINT_TYPE_NOT_SUPPORTED 201
-#define XDEBUG_ERROR_BREAKPOINT_INVALID            202
-#define XDEBUG_ERROR_BREAKPOINT_NO_CODE            203
-#define XDEBUG_ERROR_BREAKPOINT_INVALID_STATE      204
-#define XDEBUG_ERROR_NO_SUCH_BREAKPOINT            205
-#define XDEBUG_ERROR_EVALUATING_CODE               206
-#define XDEBUG_ERROR_INVALID_EXPRESSION            207 /* unused */
-
-#define XDEBUG_ERROR_PROPERTY_NON_EXISTENT         300
-#define XDEBUG_ERROR_PROPERTY_NON_EXISTANT         300 /* compatibility typo */
-#define XDEBUG_ERROR_STACK_DEPTH_INVALID           301
-#define XDEBUG_ERROR_CONTEXT_INVALID               302 /* unused */
-
-#define XDEBUG_ERROR_PROFILING_NOT_STARTED         800
-
-#define XDEBUG_ERROR_ENCODING_NOT_SUPPORTED        900
-
 typedef struct _xdebug_func {
 	zend_string *object_class;
 	zend_string *scope_class;
-	char *function;
+	zend_string *function;
+	zend_string *include_filename;
 	int   type;
 	int   internal;
 } xdebug_func;
@@ -155,31 +130,44 @@ typedef struct _function_stack_entry {
 	unsigned char      is_trampoline;
 	unsigned char      arg_done;
 
+	/* debugging properties */
+	bool has_line_breakpoints;
+
 	/* filter properties */
 	unsigned char filtered_code_coverage;
 	unsigned char filtered_stack;
 	unsigned char filtered_tracing;
 
+	/* coverage properties */
+	bool         code_coverage_init;
+	char        *code_coverage_function_name;
+	zend_string *code_coverage_filename;
+
 	/* location properties */
 	int          lineno;
 	zend_string *filename;
-	zend_string *include_filename;
 
 	/* tracing properties */
 	signed long  memory;
 	signed long  prev_memory;
 	uint64_t     nanotime;
+	bool         function_call_traced;
 
 	/* profiling properties */
 	xdebug_profile profile;
 	struct {
 		int          lineno;
 		zend_string *filename;
-		char        *funcname;
+		zend_string *function;
 	} profiler;
 
 	/* misc properties */
 	zend_op_array *op_array;
+#if PHP_VERSION_ID >= 80100
+	void           (*soap_error_cb)(int type, zend_string *error_filename, const uint32_t error_lineno, zend_string *message);
+#else
+	void           (*soap_error_cb)(int type, const char *error_filename, const uint32_t error_lineno, zend_string *message);
+#endif
 } function_stack_entry;
 
 function_stack_entry *xdebug_get_stack_frame(int nr);
@@ -219,6 +207,10 @@ typedef struct _xdebug_library_globals_t {
 	char         *log_open_timestring;
 	xdebug_str   *diagnosis_buffer;
 
+	/* Trait location map */
+	xdebug_hash  *trait_location_map;
+
+	/* Opcode overrite handlers */
 	user_opcode_handler_t          original_opcode_handlers[256];
 	xdebug_multi_opcode_handler_t *opcode_multi_handlers[256];
 	xdebug_set                    *opcode_handlers_set;
@@ -292,6 +284,12 @@ int xdebug_lib_set_start_upon_error(char *value);
 int xdebug_lib_start_upon_error(void);
 int xdebug_lib_get_start_upon_error(void);
 
+#if __linux__
+# define XDEBUG_CONTROL_SOCKET_OFF        1
+# define XDEBUG_CONTROL_SOCKET_TIME       4
+int xdebug_lib_set_control_socket_granularity(char *value);
+#endif
+
 const char *xdebug_lib_mode_from_value(int mode);
 
 void xdebug_lib_set_active_data(zend_execute_data *execute_data);
@@ -319,8 +317,10 @@ int xdebug_call_original_opcode_handler_if_set(int opcode, XDEBUG_OPCODE_HANDLER
 char *xdebug_lib_get_output_dir(void);
 
 void xdebug_llist_string_dtor(void *dummy, void *elem);
-char* xdebug_wrap_location_around_function_name(const char *prefix, zend_op_array *opa, char *fname);
-char* xdebug_wrap_closure_location_around_function_name(zend_op_array *opa, char *fname);
 
-void xdebug_lib_register_compiled_variables(function_stack_entry *fse, zend_op_array *op_array);
+zend_string *xdebug_get_trait_scope(const char *function);
+zend_string* xdebug_wrap_location_around_function_name(const char *prefix, zend_op_array *opa, zend_string *fname);
+zend_string* xdebug_wrap_closure_location_around_function_name(zend_op_array *opa, zend_string *fname);
+
+void xdebug_lib_register_compiled_variables(function_stack_entry *fse);
 #endif
