@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2021 Derick Rethans                               |
+   | Copyright (c) 2002-2022 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,7 +14,7 @@
    +----------------------------------------------------------------------+
  */
 
-#include "php.h"
+#include "lib/php-header.h"
 #include "ext/standard/php_string.h"
 #include "Zend/zend_closures.h"
 #if PHP_VERSION_ID >= 80100
@@ -35,19 +35,15 @@ static int xdebug_array_element_export(zval *zv_nptr, zend_ulong index_key, zend
 		if (HASH_KEY_IS_NUMERIC(hash_key)) { /* numeric key */
 			xdebug_str_add_fmt(str, XDEBUG_INT_FMT " => ", index_key);
 		} else { /* string key */
-			zend_string *tmp, *tmp2;
+			zend_string *tmp_zstr;
+
+			tmp_zstr = php_addcslashes(hash_key, (char*) "'\\\0..\37", 7);
 
-			tmp = php_str_to_str(ZSTR_VAL(hash_key), ZSTR_LEN(hash_key), (char*) "'", 1, (char*) "\\'", 2);
-			tmp2 = php_str_to_str(ZSTR_VAL(tmp), ZSTR_LEN(tmp), (char*) "\0", 1, (char*) "\\0", 2);
-			if (tmp) {
-				zend_string_release(tmp);
-			}
 			xdebug_str_addc(str, '\'');
-			if (tmp2) {
-				xdebug_str_add_zstr(str, tmp2);
-				zend_string_release(tmp2);
-			}
+			xdebug_str_add_zstr(str, tmp_zstr);
 			xdebug_str_add_literal(str, "' => ");
+
+			zend_string_release(tmp_zstr);
 		}
 		xdebug_var_export_line(zv, str, level + 2, debug_zval, options);
 		xdebug_str_add_literal(str, ", ");
@@ -72,9 +68,7 @@ static int xdebug_object_element_export(zval *object, zval *zv_nptr, zend_ulong
 			char       *prop_class_name;
 			const char *modifier;
 
-#if PHP_VERSION_ID >= 70400
 			property_type = xdebug_get_property_type(object, zv_nptr);
-#endif
 			property_name = xdebug_get_property_info((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), &modifier, &prop_class_name);
 			xdebug_str_add(str, modifier, 0);
 			if (property_type) {
@@ -111,6 +105,7 @@ static int xdebug_object_element_export(zval *object, zval *zv_nptr, zend_ulong
 	return 0;
 }
 
+#if PHP_VERSION_ID < 80200
 static void handle_closure(xdebug_str *str, zval *obj)
 {
 	const zend_function *closure_function;
@@ -119,17 +114,13 @@ static void handle_closure(xdebug_str *str, zval *obj)
 		return;
 	}
 
-#if PHP_VERSION_ID >= 80000
 	closure_function = zend_get_closure_method_def(Z_OBJ_P(obj));
-#else
-	closure_function = zend_get_closure_method_def(obj);
-#endif
 
 	xdebug_str_add_literal(str, "virtual $closure = \"");
 
 	if (closure_function->common.scope) {
 		if (closure_function->common.fn_flags & ZEND_ACC_STATIC) {
-			xdebug_str_add_zstr(str, closure_function->common.scope->name);
+			xdebug_str_add(str, ZSTR_VAL(closure_function->common.scope->name), 0);
 			xdebug_str_add_literal(str, "::");
 		} else {
 			xdebug_str_add_literal(str, "$this->");
@@ -139,13 +130,11 @@ static void handle_closure(xdebug_str *str, zval *obj)
 
 	xdebug_str_add_literal(str, "\", ");
 }
+#endif
 
 void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_zval, xdebug_var_export_options *options)
 {
 	HashTable *myht;
-#if PHP_VERSION_ID < 70400
-	int       is_temp;
-#endif
 	zend_ulong num;
 	zend_string *key;
 	zval *val;
@@ -190,7 +179,7 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 			break;
 
 		case IS_DOUBLE:
-			xdebug_str_add_fmt(str, "%.*G", (int) EG(precision), Z_DVAL_P(*struc));
+			xdebug_str_add_fmt(str, "%.*H", (int) EG(precision), Z_DVAL_P(*struc));
 			break;
 
 		case IS_STRING: {
@@ -199,28 +188,16 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 			} else if ((size_t) Z_STRLEN_P(*struc) <= (size_t) options->max_data) {
 				zend_string *tmp_zstr;
 
-#if PHP_VERSION_ID >= 70300
 				tmp_zstr = php_addcslashes(Z_STR_P(*struc), (char*) "'\\\0..\37", 7);
-#else
-				tmp_zstr = php_addcslashes(Z_STR_P(*struc), 0, (char*) "'\\\0..\37", 7);
-#endif
+
 				xdebug_str_addc(str, '\'');
 				xdebug_str_add_zstr(str, tmp_zstr);
 				xdebug_str_addc(str, '\'');
 
 				zend_string_release(tmp_zstr);
 			} else {
-#if PHP_VERSION_ID >= 70300
 				zend_string *tmp_zstr = php_addcslashes_str(ZSTR_VAL(Z_STR_P(*struc)), options->max_data, (char*) "'\\\0..\37", 7);
-#else
-				zend_string *tmp_zstr = NULL;
-				zend_string *truncated_zstr = zend_string_copy(Z_STR_P(*struc));
 
-				truncated_zstr = zend_string_truncate(truncated_zstr, options->max_data, 0);
-				tmp_zstr = php_addcslashes(truncated_zstr, 0, (char*) "'\\\0..\37", 7);
-
-				zend_string_release(truncated_zstr);
-#endif
 				xdebug_str_addc(str, '\'');
 				xdebug_str_add_zstr(str, tmp_zstr);
 				xdebug_str_add_literal(str, "\'...");
@@ -293,18 +270,16 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 			}
 #endif
 
-#if PHP_VERSION_ID >= 70400
 			myht = xdebug_objdebug_pp(struc, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
-#else
-			myht = xdebug_objdebug_pp(struc, &is_temp, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
-#endif
 
 			if (!myht || !xdebug_zend_hash_is_recursive(myht)) {
 				xdebug_str_add_literal(str, "class ");
 				xdebug_str_add(str, ZSTR_VAL(Z_OBJCE_P(*struc)->name), 0);
 				xdebug_str_add_literal(str, " { ");
 
+#if PHP_VERSION_ID < 80200
 				handle_closure(str, *struc);
+#endif
 
 				if (myht && (level <= options->max_depth)) {
 					options->runtime[level].current_element_nr = 0;
@@ -318,23 +293,23 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 					} ZEND_HASH_FOREACH_END();
 
 					xdebug_zend_hash_apply_protection_end(myht);
-
-					/* Remove the ", " at the end of the string */
-					if (myht->nNumOfElements > 0) {
-						xdebug_str_chop(str, 2);
-					}
 				} else {
 					xdebug_str_add_literal(str, "...");
 				}
+				/* Remove the ", " and "; " at the end of the string */
+				if (
+					str->l > 2 &&
+					((str->d[str->l - 2] == ',') || (str->d[str->l - 2] == ';' )) &&
+					(str->d[str->l - 1] == ' ')
+				) {
+					xdebug_str_chop(str, 2);
+				}
+
 				xdebug_str_add_literal(str, " }");
 			} else {
 				xdebug_str_add_literal(str, "...");
 			}
-#if PHP_VERSION_ID >= 70400
 			zend_release_properties(myht);
-#else
-			xdebug_var_maybe_destroy_ht(myht, is_temp);
-#endif
 			break;
 		}
 
