@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2021 Derick Rethans                               |
+   | Copyright (c) 2002-2022 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,7 +14,7 @@
    +----------------------------------------------------------------------+
  */
 
-#include "php.h"
+#include "lib/php-header.h"
 #include "ext/standard/php_string.h"
 #include "Zend/zend_closures.h"
 #if PHP_VERSION_ID >= 80100
@@ -102,9 +102,7 @@ static int xdebug_object_element_export_text_ansi(zval *object, zval *zv_nptr, z
 			const char *modifier;
 			xdebug_str *property_type = NULL;
 
-#if PHP_VERSION_ID >= 70400
 			property_type = xdebug_get_property_type(object, zv_nptr);
-#endif
 
 			property_name = xdebug_get_property_info((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), &modifier, &class_name);
 			xdebug_str_add_fmt(
@@ -134,6 +132,7 @@ static int xdebug_object_element_export_text_ansi(zval *object, zval *zv_nptr, z
 	return 0;
 }
 
+#if PHP_VERSION_ID < 80200
 static void handle_closure(xdebug_str *str, zval *obj, int level, int mode)
 {
 	const zend_function *closure_function;
@@ -142,11 +141,7 @@ static void handle_closure(xdebug_str *str, zval *obj, int level, int mode)
 		return;
 	}
 
-#if PHP_VERSION_ID >= 80000
 	closure_function = zend_get_closure_method_def(Z_OBJ_P(obj));
-#else
-	closure_function = zend_get_closure_method_def(obj);
-#endif
 
 	xdebug_str_add_fmt(
 		str, "%*s%s%svirtual%s $closure =>\n%*s\"",
@@ -169,16 +164,13 @@ static void handle_closure(xdebug_str *str, zval *obj, int level, int mode)
 	xdebug_str_add_zstr(str, closure_function->common.function_name);
 	xdebug_str_add_fmt(str, "%s\"\n", ANSI_COLOR_RESET);
 }
-
+#endif
 
 static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode, int level, int debug_zval, xdebug_var_export_options *options)
 {
 	HashTable *myht;
 	char*     tmp_str;
 	int       tmp_len;
-#if PHP_VERSION_ID < 70400
-	int       is_temp;
-#endif
 	zend_ulong num;
 	zend_string *key;
 	zval *val;
@@ -225,7 +217,7 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 			break;
 
 		case IS_DOUBLE:
-			xdebug_str_add_fmt(str, "%sdouble%s(%s%.*G%s)", ANSI_COLOR_BOLD, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_DOUBLE, (int) EG(precision), Z_DVAL_P(*struc), ANSI_COLOR_RESET);
+			xdebug_str_add_fmt(str, "%sdouble%s(%s%.*H%s)", ANSI_COLOR_BOLD, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_DOUBLE, (int) EG(precision), Z_DVAL_P(*struc), ANSI_COLOR_RESET);
 			break;
 
 		case IS_STRING: {
@@ -235,11 +227,7 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 			zend_string *i_string = zend_string_init(Z_STRVAL_P(*struc), Z_STRLEN_P(*struc), 0);
 			zend_string *tmp_zstr;
 
-#if PHP_VERSION_ID >= 70300
 			tmp_zstr = php_addcslashes(i_string, (char*) pattern, pattern_len);
-#else
-			tmp_zstr = php_addcslashes(i_string, 0, (char*) pattern, pattern_len);
-#endif
 
 			tmp_str = estrndup(tmp_zstr->val, tmp_zstr->len);
 			tmp_len = tmp_zstr->len;
@@ -284,7 +272,7 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 					} ZEND_HASH_FOREACH_END();
 
 					xdebug_zend_hash_apply_protection_end(myht);
-				} else {
+				} else if (myht->nNumOfElements > 0) {
 					xdebug_str_add_fmt(str, "%*s...\n", (level * 2), "");
 				}
 				xdebug_str_add_fmt(str, "%*s}", (level * 2) - 2 , "");
@@ -328,11 +316,7 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 			}
 #endif
 
-#if PHP_VERSION_ID >= 70400
-			myht = xdebug_objdebug_pp(struc);
-#else
-			myht = xdebug_objdebug_pp(struc, &is_temp);
-#endif
+			myht = xdebug_objdebug_pp(struc, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
 
 			if (!myht || !xdebug_zend_hash_is_recursive(myht)) {
 				xdebug_str_add_fmt(
@@ -343,7 +327,9 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 					ANSI_COLOR_LONG, myht ? myht->nNumOfElements : 0, ANSI_COLOR_RESET
 				);
 
+#if PHP_VERSION_ID < 80200
 				handle_closure(str, *struc, level, mode);
+#endif
 
 				if (myht && (level <= options->max_depth)) {
 					options->runtime[level].current_element_nr = 0;
@@ -357,18 +343,14 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 					} ZEND_HASH_FOREACH_END();
 
 					xdebug_zend_hash_apply_protection_end(myht);
-				} else {
+				} else if (myht->nNumOfElements > 0) {
 					xdebug_str_add_fmt(str, "%*s...\n", (level * 2), "");
 				}
 				xdebug_str_add_fmt(str, "%*s}", (level * 2) - 2, "");
 			} else {
 				xdebug_str_add_fmt(str, "%*s...\n", (level * 2), "");
 			}
-#if PHP_VERSION_ID >= 70400
 			zend_release_properties(myht);
-#else
-			xdebug_var_maybe_destroy_ht(myht, is_temp);
-#endif
 			break;
 		}
 
